; ==============================================================================
#include        "a-mac.asi"
#include        "a-defs.asi"

; ------------------------------------------------------------------------------
#include        "via.h"

; ------------------------------------------------------------------------------
ROMO8080        JMP   ROM_ENTRY
;               JMP   ROM_PREAD         ; joystick position
;               JMP   ROM_COUT          ; character output
;               JMP   ROM_BELL          ; outputs ctrl-g
;               JMP   ROM_BELL2         ; 60hz bell
;               JMP   ROM_OLDRST        ; set screen mode and init
;               JMP   ROM_BASCALC       ; calculate base address
;               JMP   ROM_PRBYTE        ; prints byte as 2 hex digits
;               JMP   ROM_RDKEY         ; read keyboard input
;               JMP   ROM_CLREOL        ; say end of line
;               JMP   ROM_SETTXT        ; set text mode
;               JMP   ROM_CROUT         ; print cr
;               JMP   ROM_HOME          ; home
;               JMP   ROM_GETKEY        ; keyboard buffer

; ------------------------------------------------------------------------------
_XHOST          BYT   "raxiss.com:80",0

_XDISK_A        BYT   "cpm2.dsk",0                  ; FDD 0
_XDISK_B        BYT   "app.dsk",0                   ; FDD 1
_XDISK_C        BYT   "mpm.dsk",0                   ; FDD 2
_XDISK_D        BYT   "tp1.dsk",0                   ; FDD 3
_XDISK_E        BYT   "tp3.dsk",0                   ; FDD 4
_XDISK_F        BYT   0                             ; FDD 5
_XDISK_G        BYT   0                             ; FDD 6
_XDISK_H        BYT   0                             ; FDD 7

_XDISKLO        BYT   <_XDISK_A,<_XDISK_B,<_XDISK_C,<_XDISK_D
                BYT   <_XDISK_E,<_XDISK_F,<_XDISK_G,<_XDISK_H
_XDISKHI        BYT   >_XDISK_A,>_XDISK_B,>_XDISK_C,>_XDISK_D
                BYT   >_XDISK_E,>_XDISK_F,>_XDISK_G,>_XDISK_H

_XDISK          BYT   $00

; ------------------------------------------------------------------------------
ROM_DSKIO       CMP   #READCMD
                BNE   RMDEND
                LDA   DSKTRACK
                CMP   OLDTRACK
                BEQ   RMDEND
                STA   OLDTRACK

                LDA   !DSKSELECT
                BMI   RMDEND
                AND   #$07
                CMP   #$05
                BCS   RMDEND

                STA   _XDISK
                LDX   DSKTRACK
                LDY   DSKSECTOR
              ; STA   ODEBUG

                JSR   ROM_HTTPREAD

RMDEND          LDA   #$00
                RTS

; ------------------------------------------------------------------------------
ROM_HTTPREAD    LDA   #<_XHOST
                STA   !__http_srv
                LDA   #>_XHOST
                STA   !__http_srv+1

                LDY   _XDISK
                LDA   _XDISKLO,Y
                STA   !__http_url
                LDA   _XDISKHI,Y
                STA   !__http_url+1

                LDA   #<TRACKBUFFER
                STA   !__http_ptr
                LDA   #>TRACKBUFFER
                STA   !__http_ptr+1

                LDA   #$11
                STA   !__http_len+1
                LDA   #$20
                STA   !__http_len

                LDA   #$00
                STA   !__http_off+3
                STA   !__http_off+2
                STA   !__http_off+1
                LDA   DSKTRACK
                STA   !__http_off

                JMP   __http_getblob
;               RTS

; ------------------------------------------------------------------------------
ROM_OLDRST      ; JMP   ROM_ENTRY         ; RESTART
; ------------------------------------------------------------------------------
ROM_ENTRY       SEI
                LDA   #$7F
                STA   via_ier
                LDA   #$00
                STA   via_ifr

                STA   KEY
                JSR   ROM_HOME

RUNNER          LDA   STUB8080_ADDRESS
                STA   REL
                JMP   I8800

; ------------------------------------------------------------------------------
ROM_GETKEY      STX   TMPX
                STY   TMPY
                JSR   _key_getc
                BEQ   NEWKEY
                CMP   #$60
                BCC   CHKCTL            ;NOCASE
                AND   #$DF

CHKCTL          CMP   #$0C              ; HANDLE ^L
                BNE   NOCASE
                JSR   ROM_HOME
                JSR   CURSORON
                JMP   NOKEY

NOCASE          ORA   #$80
SAMEKEY         CMP   KEY
                BNE   NEWKEY
NOKEY           LDA   #$00
NEWKEY          STA   KEY
                LDY   TMPY
                LDX   TMPX
                LDA   KEY
                RTS

; ------------------------------------------------------------------------------
ROM_BELL        JMP   _keyclick1        ; OUTPUTS CTRL-G
; ------------------------------------------------------------------------------
ROM_BELL2       JMP   _keyclick2        ; 60HZ BELL

; ------------------------------------------------------------------------------
; READ KEY SUBROUTINE: MAKES CURRENT CHAR FLASH; & PUTS IT IN (A) & (CH) IN (Y)
; ------------------------------------------------------------------------------
ROM_RDKEY       RTS                     ; READ KEYBOARD INPUT

; ------------------------------------------------------------------------------
ROM_PREAD       LDY   #$FF              ; POSITION OF A HAND CONTROL
                RTS

; ------------------------------------------------------------------------------
ROM_COUT        PHA                     ; CHARACTER OUTPUT
                STY   TMPY              ; SAV Y-REG
                JSR   VIDOUT            ; OUTPUT A-REG AS ASCII
                JSR   CURSORON
                LDY   TMPY              ; AND Y-REG
                PLA                     ; RESTORE A-REG
                RTS

; ------------------------------------------------------------------------------
ROM_BASCALC     PHA                     ; CALCULATE BASE ADDRESS
                LDA   #<WNDA
                STA   CURPOS
                LDA   #>WNDA
                STA   CURPOS+1
                LDA   #$00
                STA   TCNT
                PLA
                ASL
                ROL   TCNT
                ASL
                ROL   TCNT
                ASL
                ROL   TCNT
                PHA
                CLC
                ADC   CURPOS
                STA   CURPOS
                LDA   TCNT
                ADC   CURPOS+1
                STA   CURPOS+1
                PLA
                ASL
                ROL   TCNT
                ASL
                ROL   TCNT
                CLC
                ADC   CURPOS
                STA   CURPOS
                LDA   TCNT
                ADC   CURPOS+1
                STA   CURPOS+1
                RTS

; ------------------------------------------------------------------------------
; PRINT A-REG AS TWO-DIGIT HEX NUMBER:
; ------------------------------------------------------------------------------
ROM_PRBYTE      PHA                     ; PUSH ACCUMULATOR (A); SAVE ON STACK
                LSR                     ; SHIFT HIGH NIBBLE
                LSR                     ; ... TO LOW NIBBLE
                LSR                     ; THESE 4 SHIFTS ALSO CLEAR HIGH NIBBLE
                LSR                     ; NOW LOW NIBBLE IS WHAT HIGH NIBBLE WAS
                JSR   PRHEXZ            ; PRINT HI-DIGIT IN LO-NIBBLE AS HEX NUMBER
                PLA                     ; PULL ACCUMULATOR (A); RETRIEVE FROM STACK
PRHEX           AND   #%00001111        ; MASK OFF HIGH NIBBLE & KEEP LOW NIBBLE
PRHEXZ          ORA   #'0'              ;| $80        ; ASSURE BYTE IS HEX NUMBER CHR [0~9 & A~F]
                CMP   #':'              ;| $80        ; ASSURE BYTE IS DEC NUMBER CHR [0~9]<(":")
                BCC   PRCOUT            ; PRINT # NOW IF DEC NUMBER CHR [0~9]<(":")
                ADC   #6                ; ELSE ASSURE LETTER [A~F]>=[(":")+6+(C=1)]
PRCOUT          JMP   ROM_COUT          ; PRINT ACCUMULATOR (A) TO OUTPUT DEVICE

; ------------------------------------------------------------------------------
ROM_CLREOL      :.(                     ; SAY END OF LINE
                LDY   HPOS              ; CURSOR H INDEX
                LDA   #' '
CLEOL2          STA   (CURPOS),Y        ; STORE BLANKS FROM 'HERE'
                INY                     ; TO END OF LINES (#40)
                CPY   #WNDW
                BCC   CLEOL2
                JMP   CURSORON
;               RTS
                :.)

; ------------------------------------------------------------------------------
ROM_CROUT       JSR   CURSOROFF
ROM_CROUT2      INC   VPOS
                LDA   VPOS
                CMP   #WNDH
                BCC   RCROUT
                JSR   SCROLL
                LDA   #WNDH-1
RCROUT          JSR   ROM_BASCALC
                LDA   #$00
                STA   HPOS
                JMP   CURSORON
;               RTS

; ------------------------------------------------------------------------------
; SET TEXT MODE
ROM_SETTXT
; ------------------------------------------------------------------------------
; HOME
ROM_HOME        JSR   HOMEPOSCP
                LDX   #WNDH
ROMHLY          LDY   #WNDW-1
                LDA   #' '
ROMHLX          STA   (CURPOS),Y
                DEY
                BPL   ROMHLX
                CLC
                LDA   #WNDW
                ADC   CURPOS
                STA   CURPOS
                BCC   ROMHSK
                INC   CURPOS+1
ROMHSK          DEX
                BNE   ROMHLY
                TXA                     ; X=0
                TAY                     ; RETURN A=X=Y=0
;               JMP   HOMEPOS           ; NOTE: falltrough
; --------------^^^-------------------------------------------------------------
HOMEPOS         JSR   HOMEPOSCP
                LDA   #$00
                STA   HPOS
                STA   VPOS
                STA   TINV
                RTS
HOMEPOSCP       LDA   #<WNDA
                STA   CURPOS
                LDA   #>WNDA
                STA   CURPOS+1
                RTS

; ------------------------------------------------------------------------------
CURSORON        :.(
;                 PHA
;                 TYA
;                 PHA
                LDY   HPOS
                LDA   #' '|$80
                STA   (CURPOS),Y
;                 PLA
;                 TAY
;                 PLA
                RTS
                :.)

; ------------------------------------------------------------------------------
CURSOROFF       :.(
;                 PHA
;                 TYA
;                 PHA
                LDY   HPOS
                LDA   #' '
                STA   (CURPOS),Y
;                 PLA
;                 TAY
;                 PLA
                RTS
                :.)

; ------------------------------------------------------------------------------
SCROLL          :.(
                LDA   #<WNDA+WNDW
                STA   TSRC
                LDA   #>WNDA+WNDW
                STA   TSRC+1
                LDA   #<WNDA
                STA   TDST
                LDA   #>WNDA
                STA   TDST+1
                LDA   #WNDH-1
                STA   TCNT
                LDY   #$00
LOOPY           LDX   #WNDW
LOOPX           LDA   (TSRC),Y
                STA   (TDST),Y

                INC   TSRC
                BNE   SKIP1
                INC   TSRC+1
SKIP1           INC   TDST
                BNE   SKIP2
                INC   TDST+1
SKIP2           DEX
                BNE   LOOPX
                DEC   TCNT
                BNE   LOOPY

                LDY   #WNDW-1
                LDA   #' '

LOOPZ           STA   (TDST),Y
                DEY
                BPL   LOOPZ

                RTS
                :.)

; ------------------------------------------------------------------------------
VIDOUT          AND   #$7F              ; OUTPUT A-REG AS TEXT
                CMP   #$0D              ; PLACE IN SCREEN MEMORY
                BNE   CHKFF             ; OR PROCESS CONTROL CHARACTER
                JMP   ROM_CROUT
CHKFF           CMP   #$0C              ; FF
                BNE   CHKBS
                JMP   ROM_HOME
CHKBS           CMP   #$5F              ; BS
                BNE   STORADV
                JMP   VIDBS
STORADV         LDY   HPOS              ; SET Y-INDEX TO CURSOR HORIZONTAL (HPOS)
                ORA   TINV
                STA   (CURPOS),Y        ; PUT CHAR (A) IN (BASE ADRS) LINE AT (HPOS)
                INY                     ; ADVANCE CURSOR HORIZONTAL (MOVE RIGHT)
                CPY   #WNDW             ; IS CURSOR HORIZONTAL PAST WINDOW WIDTH?
                BCC   VIDOUTRTS
                JMP   ROM_CROUT2        ; YES, RTN CURSOR (CR) TO NEXT LINE
VIDOUTRTS       STY   HPOS              ; GET CURSOR HORIZONTAL
                RTS

; ------------------------------------------------------------------------------
VIDBS           BIT   HPOS              ; BACKSPACE ...
                BEQ   VIDBSRTS
                JSR   CURSOROFF
                DEC   HPOS
                JMP   CURSORON
VIDBSRTS        RTS

; ------------------------------------------------------------------------------
; Y:A = string ptr
; STROUT          STA   $00
;                 STY   $01
;                 LDY   #$00
;
; CHAROUT         LDA   ($00),Y
;                 BEQ   DONESTR           ; char == 00, EOL
;                 JSR   ROM_COUT          ; print char
;                 INY                     ; next char
;                 BNE   CHAROUT
;                 INC   $01
;                 BNE   CHAROUT
; DONESTR         RTS
