; ------------------------------------------------------------------------------
; APPLE-80 - an 8080 simulator-debug package Copyright (c) 1979 by Dann Mccreary
; SIM8800  - Altair 8800 simulator by Charles Mangin, 2019
; ORIC8080 - Altair 8800 simulator for Oric by iss@raxiss
; ------------------------------------------------------------------------------

; ------------------------------------------------------------------------------
#include        "a-mac.asi"
#include        "a-defs.asi"

; ==============================================================================
I8800           SEI
                CLD                     ; CLEAR DECIMAL MODE
                JSR   ROM_HOME
                JSR   HELLO
                JSR   REGSETUP

MAINLOOP
;               JSR   PCHFIX
;               JSR   DUMPREGS

;               LDX   PCH
;               LDY   PCL
;               LDA   #00
;               STA   ODEBUG

                JSR   SIM80
                JMP   MAINLOOP

; ------------------------------------------------------------------------------
SIM80           CLC
                LDA   PCH
                ADC   REL               ; #$10
                STA   PCH

                JSR   MAIN              ; execute OPCODE

                SEC
                LDA   PCH
                SBC   REL               ; #$10
                STA   PCH               ; store resulting PCH

                LDA   PSW               ; processor status
                AND   #$D7              ; AND 1101 0111 - clear MI and HLTA bits
                ORA   #$02              ; OR 0000 0010 - set WO bit
                STA   PSW

;               CLI                     ; clear interrupts
                JMP   SIMDSK            ; J/R, operate virtual disk drive
;               RTS

; ------------------------------------------------------------------------------
; EXECUTE ONE 8080 OPCODE
MAIN            SEI                     ; DON`T ALLOW INTERRUPTS
                LDX   #$00              ; LOAD INDEX

                LDA   SIMH
                CLC
STASIMM2        ADC   REL               ; #$10
                STA   SIMH
STASIMM         LDA   (SIML,X)          ; FETCH MEMORY *** from (HL) register pair ***
                STA   SIMM              ; crashes II if SIMH is C0 softswitch area
DECSIMH         SEC                     ; then subtract #$10... oof.
                LDA   SIMH
                SBC   REL               ; #$10
                STA   SIMH
LDAPC           LDA   (PCL,X)           ; FETCH INSTRUCTION from real $1000-$8fff

                STA   OPCODE
                EOR   #$76
                TAX
                LDY   #$01
                LDA   PNT
                AND   INTE              ; IS THIS A HALT?
                BPL   CKINT
                STY   INTE
                STY   PNT
                ORA   #$C7
                STA   OPCODE
                DEY

; CHECK FOR INTERRUPT
CKINT           LDA   INTE
                BNE   NINT
                DEC   INTE

; ENABLE INTERRUPT
NINT            TXA
                BNE   NOHLT
                DEY

; HALT?
NOHLT           TYA
                BEQ   ITR
                JSR   INCPC

; SET COUNTER/INDEX
ITR             LDX   #$02
                LDA   OPCODE
DIV             PHA
                AND   #$07
                EOR   #$07
                STA   ZERO,X
                PLA
                LSR
                LSR
                LSR
                DEX
                BNE   DIV
                STX   FLAG
                LSR
                BCS   IMI
                BEQ   FIRST
IMARL           LDA   #$11              ; no, arilog
                ADC   ZERO+$01          ; SIMA
                TAX
                LDA   #$00              ; set dest=A
                STA   ZERO+$01          ; SIMA
IMI             BEQ   MOVE
SECOND          LDA   #$07              ; offset for 2nd (page?)
FIRST           ADC   SPEED
                TAX                     ; Transfer to Index
                INX

MOVE            TXA                     ; SET HIGH ADDRESS
                ASL
                TAX
                LDA   XFERTBL,X         ; GET LO ADOR
                INX
                PHA                     ; SET LO ADDRESS
                LDA   XFERTBL,X
                PHA
                LDX   SPEED             ; GET SOURCE INDEX
                LDY   SIMA,X            ; GET SOURCE DATA IN Y (label was NOFETCH:)
                LDX   ZERO+$01          ; GET DEST INDEX
                LDA   SIMA,X
                STA   ZERO              ; SAVE DESTINATION DATA
                LDA   #$44              ; SET 6502 STATUS
                PHA
                TXA                     ; DEST IND, IN A & X
                RTI                     ; GO INTERPRET

; ------------------------------------------------------------------------------
IN              LSR
                BEQ   XCHG
                BCC   INPUT
                TAY
                JSR   CALL65
                STA   SIMA
                RTS

; ------------------------------------------------------------------------------
; CALL 6502 SUBROUTINE
CALL65          LDA   (PCL),Y
                PHA
                DEY
                BPL   CALL65
                JSR   DBLINC
                LDA   #$00
                PHA
                LDA   SIMA
                RTI

; ------------------------------------------------------------------------------
XCHG            JSR   RPSCR
                LDX   #$04
                LDY   #$02
                JSR   RPRP
                LDX   #$04
                JMP   SCRRP

; ------------------------------------------------------------------------------
XTHL            JSR   RPSCR
                JSR   SPPNT
                LDX   #$02
                JSR   MEMRP
                LDX   #$13
                BNE   RPMEM

; ------------------------------------------------------------------------------
IO              LSR                     ; which inst?
                BNE   LBL
                STX   INTE              ; set/clear INTE
                RTS

; ------------------------------------------------------------------------------
LBL             LDX   #$02              ; point to HL
                BCC   IN                ; is it input
                LSR                     ; no, is it xthl?
                BEQ   XTHL              ; yes
                BCS   JMPJUMPUN         ; is it jump?
OUTPUT          DEC   FLAG              ; no, flag an output
INPUT           JSR   IMM               ; get port #
                LDA   SRC               ; SRC = port number

                AND   #$0F              ; clear high bits (FF => 0F) because why keep up with 256 ports when we only need 16?
                ASL                     ; double index for hi/lo byte in table
                TAX                     ; port table offset in X
                LDY   #$02
IOPLP           LDA   IOTBLOUT,X        ; get IO address
                STA   DESTDA-1,Y        ; store in pointer
                INX                     ; bump index
                DEY                     ; decrement counter
                BNE   IOPLP

;               INY                     ; OPTIMIZED: point to data diriction reg
;PNTPRT         DEY                     ; OPTIMIZED: point to port

                LSR   FLAG              ; FLAG is FF on OUTPUT, 00 on INPUT
                BCC   IOIN              ; do input if carry clear.
                LDA   SIMA              ; - if carry set, OUTPUT
IOOUT           STA   (DESTDA),Y        ; OUTPUT TO PORT
                JSR   SIOOUT            ; TELL SIO I HAVE PUT A BYTE TO SEND
                BCS   IOEX              ; TODO: should return with Carry set ???

IOIN            INC   DESTDA            ; switch from PORT-OUT to PORT-IN
                TYA
                PHA
                JSR   SIOIN             ; TELL SIO I WANT A BYTE - Y=PORT
                PLA
                TAY
                LDA   (DESTDA),Y        ; input from port
                STA   SIMA              ; store in SIMA
IOEX            RTS

JMPJUMPUN       JMP   JUMPUN            ; spaghetti

; ------------------------------------------------------------------------------
ARIM            JSR   IMM
                CLC
                JMP   IMARL

JMPCALLUN       JMP   CALLUN            ; spaghetti

; ------------------------------------------------------------------------------
PUSH            LSR                     ; IS THIS A PUSH?
                BCS   SKP               ; YES, GO PUSH
                CMP   #$03              ; NO, IS THIS A CALL?
                BEQ   JMPCALLUN         ; YES, GO CALL
                JMP   UNDEF             ; NO, UNDEFINED

SKP             ASL                     ; MAKE AN INDEX
                TAX                     ; IS IT PUSH PSW?
                BNE   PUSHT             ; NO, KEEP INDEX
                DEX                     ; YES, ADJUST INDEX

PUSHT           TXA                     ; TEMP SAVE - x=0A on set return, gets PCL/H
                PHA
                JSR   DECSP             ; DECREMENT STACK POINTER
                JSR   SPPNT             ; IT BECOMES POINTER
                PLA                     ; RECOVER TEMP SAVE
                TAX

RPMEM           LDY   #$01              ; CLEAR INDEX

; compare to 8f, THEN add 10 if necessary

;               CLC                     ; not needed
                LDA   DESTDA,Y          ; $04 + $12 = $16 DESTDA
                CPX   #$02
                BNE   RPLP

; E5 PUSH H AXY=02/02/01
; 22 SHLD   AXY=02/01/01

                LDA   OPCODE            ; is it PUSH H or SHLD? <-- corner case!
                CMP   #$22
                BNE   RPLP

                CLC                     ; ADD 10 to destination on SHLD
                LDA   DESTDA,Y
                ADC   REL               ; #$10
                STA   DESTDA,Y

RPLP            LDA   SIMM,X            ; GET NEXT RP DATA
; @WTF?         PHA                     ; hang onto it
; @WTF?         LDA   DESTDA+1
; @WTF?         PLA                     ; get RP data back
                STA   (DESTDA),Y        ; STORE IN MEMORY

RPLP2           DEX
                DEY
                BEQ   RPLP

                CPX   #$00
                BNE   RTS_
                INY
                INY

                SEC
                LDA   DESTDA,Y          ; $04 + $12 = $16 DESTDA
                SBC   REL               ; #$10
                STA   DESTDA,Y

RTS_            RTS

; ------------------------------------------------------------------------------
JMP_            CLV                     ; INDICATE A JUMP

CALL            JSR   CONDIT            ; TEST CONDITION
                BEQ   CALLUN            ; MET
                JMP   DBLINC            ; NOT MET, increment PC

; ------------------------------------------------------------------------------
JUMPUN          CLV                     ; INDICATE A JUMP
CALLUN          PHP                     ; preserve overflow, etc
                JSR   PCPNT             ; GET NEXT 2 BYTES
                LDX   #$13              ; INTO SCRATCH
                JSR   MEMRP

                CLC
                LDA   SRC+1
                ADC   REL               ; #$10              ; CLEARS OVERFLOW?
                STA   SRC+1

                PLP                     ; get overflow, etc.
CALLUN2         BVC   JM                ; JUMP on overflow clear
                JSR   DBLINC            ; otherwise? BUMP PC twice
SVR             LDX   #$0A              ; SAVE RETURN for RST

                SEC
                LDA   PCH
                SBC   REL               ; #$10
                STA   PCH

                JSR   PUSHT             ; ON STACK **** <--- and if SP is >9000?

                CLC
                LDA   PCH
                ADC   REL               ; #$10
                STA   PCH

JM              LDX   #$0A              ; POINT TO PC
                JMP   SCRRP             ; GIVE IT NEW ADDRESS

; ------------------------------------------------------------------------------
RETURN          JSR   CONDIT            ; TEST CONDITION
                BNE   RTS_              ; NOT MET

RETUN           LDA   SPH

RETUN2          CLC
                LDY   #$01
                LDA   (SPL),Y           ; get byte at (SPL/H)
                ADC   REL               ; #$10
                STA   (SPL),Y

RETUN3          LDX   #$0A              ; POP RETURN OFF
                BNE   POPIT             ; STACK INTO PC

RST             LDA   OPCODE            ; CONVERT INSTRUCTION
                AND   #$38              ; TO RESET VECTOR
                STA   SRC               ; STORE IT IN SCRATCH

                LDA   #$10              ; FIXME: #$10 ot REL ??? HIGH RESET VECTOR
                STA   SRC+1             ; ??? SCR+1 ???
                BCC   SVR

; ------------------------------------------------------------------------------
POP             LSR                     ; POP?
                BCC   OTH               ; NO
                ASL
                TAX                     ; POP PSW?
                BNE   POPIT             ; NO, KEEP INDEX
                DEX                     ; YES, ADJUST INDEX

POPIT           TXA                     ; TEMP SAVE INDEX
                PHA
                JSR   SPPNT             ; PULL OUT STACK DATA
                PLA                     ; BUMP SP
                TAX

                JSR   MEMRP             ; SP IS POINTER
ICISP           JSR   ISP               ; RECOVER TEMP
ISP             LDX   #$08
                JMP   INCPC2

; ------------------------------------------------------------------------------
OTH             LDX   #$02              ; POINT TO HL
                LDY   #$08              ; ASSUME SPHL
                ASL                     ; IS IT SPHL?
                BEQ   SPHL              ; YES, GO DO IT
                CMP   #$04              ; NO, WHAT IS IT?
                BEQ   UNDEF             ; UNDEFINED
                BPL   RETUN             ; RETURN
                LDY   #$0A              ; PCHL

                CLC
                LDA   SIMH
                ADC   REL               ; #$10
                STA   SIMH

SPHL            JMP   RPRP

; ------------------------------------------------------------------------------
CONDIT          LSR                     ; MAKE INDEX TO MASK. TABLE
                TAX
                LDA   MSKTBL,X          ; GET CONDITION FLAG MASK
                AND   PSW               ; IS CONDITION SET*
                BCS   RTN               ; LEAVE Z FLAG SET
                EOR   MSKTBL,X          ; OR REVERSE IT
RTN             RTS

; ------------------------------------------------------------------------------
NOP_            LDA   OPCODE            ; CHECK INSTRUCTION
                BEQ   RTN               ; Pointer says 00 == NOP

; ------------------------------------------------------------------------------
UNDEF           JSR   ROM_BELL          ; illegal/undefined opcode
                JMP   DUMPREGS          ; J/R
;               RTS

; ------------------------------------------------------------------------------
DADLXI          LSR                     ; DROP LSB
                ASL
                BNE   NSP               ; USE SP?
                LDA   #$08              ; YES, POINT TO IT

NSP             PHA                     ; SAVE POINTER a=8
                TXA                     ; GET DESTINATION INDEX
                LSR                     ; DAD?
                BCS   LXI               ; NO, GO DO LXI
DAD             PLA                     ; YES, RECOVER POINTER a=8
                TAY
                LDX   #$02              ; SET HL AS DESTINATION

                LDA   OPCODE
                CMP   #$39              ; if we`re working with stack pointer
                BNE   DAD2

                SEC
                LDA   SPH
                SBC   REL               ; #$10
                STA   SPH

DAD2            JSR   INCDEC            ; GO ADD RP & HL

                PHA
                PHP                     ; preserve carry set, etc
                LDA   OPCODE
                CMP   #$39
                BNE   CARRY2

                CLC
                LDA   SPH
                ADC   REL               ; #$10
                STA   SPH

CARRY2          PLP                     ; reset carry, etc
                PLA

CARRY           TAY                     ; SAVE RESULT
                ROL                     ; BRING IN CARRY
                EOR   FLAG              ; CHG TO BORROW IF SUBTRACT
                LSR   PSW
                LSR                     ; RESTORE CARRY(BORROW)
                ROL   PSW
                TYA                     ; RECOVER RESULT
                RTS

; ------------------------------------------------------------------------------
LXI             JSR   PCPNT             ; PC INTO PNT
                PLA                     ; RECOVER POINTER
                TAX                     ; USE TO INDEX DESTINATION RP
                JSR   MEMRP             ; MOVE IMMEDIATE DATA TO RP

; if RP is SP, add 10 to SPH

                LDA   OPCODE
                CMP   #$31
                BNE   LXI2

                CLC
                LDA   SPH
                ADC   REL               ; #$10
                STA   SPH

LXI2            JMP   DBLINC            ; BUMP PROGRAM COUNTER

; ------------------------------------------------------------------------------
LDSTR           BIT   FOUR              ; INDIRECT?
                BEQ   DIRECT            ; NO
                LSR                     ; STAX?
                BCC   LDAX              ; NO

STAX            INX

                CLC
                LDA   PSW,X
                ADC   REL               ; #$10
                STA   PSW,X             ; store updated destination address

                DEX
                LDA   SIMA              ; YES,STORE A
                STA   (PSW,X)           ; PSW = 04 + 12 = 16 = destDA
                INX

                SEC
                LDA   PSW,X
                SBC   REL               ; #$10
                STA   PSW,X

                DEX
                RTS

; ------------------------------------------------------------------------------
DIRECT          PHA                     ; TEMP SAVE
                JSR   PCPNT             ; PC TO POINTER
                JSR   DBLINC
                LDX   #$13
                JSR   MEMRP

                LDA   OPCODE
                CMP   #$2A
                BNE   DIRECT2

                CLC
                LDA   SRC+1
                ADC   REL               ; #$10
                STA   SRC+1

DIRECT2         LDX   #$11
                JSR   SCRRP
                PLA                     ; RECOVER TEMP
                LSR                     ; LOAD?
                BCS   STORE             ; NO, STORE
                ASL
                TAX
                BEQ   LDAD
                JMP   MEMRP

; ------------------------------------------------------------------------------
LDAD            LDX   #$11

LDAX            INX

                CLC
                LDA   SIMA,X
                ADC   REL               ; #$10
                STA   SIMA,X

                DEX
                LDA   (SIMA,X)
                STA   SIMA
                INX

                SEC
                LDA   SIMA,X
                SBC   REL               ; #$10
                STA   SIMA,X

                DEX
                RTS

; ------------------------------------------------------------------------------
STORE           ASL                     ; MAKE INDEX
                TAX
                BEQ   STAD              ; STORE A DIRECT?
                JMP   RPMEM             ; NO, STORE HL DIRECT SHLD

STAD            LDX   #$12              ; POINT TO POINTER
                JMP   STAX              ; jump to STAX

; ------------------------------------------------------------------------------
INXDCX          LDY   #$0C              ; SET FOR DECREMENT
                LSR
                BCC   DRP               ; YES
                LDY   #$0E              ; NO, SET FOR INCR
DRP             ASL                     ; DROP LSB
                BNE   NOR               ; FOR SP?
                LDA   #$08              ; YES, POINT TO SP
NOR             TAX                     ; USE AS INDEX
                JMP   INCDEC

; ------------------------------------------------------------------------------
DCR             SEC                     ; SET FOR DECREMENT
                DEC   FLAG
INR             LDY   #$01              ; SET FOR INCREMENT
BTH             JSR   ADDR              ; GO ADD
                JMP   STATUS2           ; SET STATUS, NOT CARRY

; ------------------------------------------------------------------------------
MVI             JSR   IMM               ; GET IMMEDIATE BYTE
                JMP   MOVE              ; MOVE IT TO DESTINATION

; ------------------------------------------------------------------------------
MOVIT           TYA                     ; GET SOURCE DATA
MVIT1           DEX                     ; IS DEST MEMORY?
                BNE   NMEM              ; NO

                INX
                PHA

                CLC
                LDA   SIML,X
                ADC   REL               ; #$10              ; add #$10
                STA   SIML,X

                PLA
                DEX
                STA   (SIML,X)          ; YES STORE IN MEMORY
                INX
                PHA

                SEC
                LDA   SIML,X
                SBC   REL               ; #$10
                STA   SIML,X

NOMOV           PLA
                DEX

NMEM            STA   SIMM,X            ; STORE IN DEST REGISTER
                RTS

; ------------------------------------------------------------------------------
ADC_            CLV                     ; FLAG W/CARRY
                BVC   ADD

CMP_            LDX   #$13              ; CHG DEST TO SCR
                BNE   SUB               ; SO ONLY PSW IS AFFECTED

SBB             CLV                     ; FLAG W/BORROW
SUB             DEC   FLAG              ; F L A G = FF FOR SUBTRACT
                SEC

ADD             JMP   ADCRY

; ------------------------------------------------------------------------------
ORA_            TYA                     ; GET SOURCE DATA
                ORA   SIMA              ; LOGICAL OR
                BCC   SOM               ; SET FOR A CLEAR AUX CRY

XRA             TYA                     ; G E T SOURCE WRD   T A
                EOR   SIMA              ; LOGICAL EXCLUSIVE OR
SOM             LDY   #$00              ; 00 WON`T SET ANY FLAGS
                BEQ   ALL               ; GO SAVE RESULT

ANA             TYA                     ; G E T SOURCE WRD   T A
                PHA                     ; SAVE IT
                ORA   SIMA              ; LOGICAL OR OF BIT
                ASL                     ; T H R E E BECOMES AUX CARRY
                AND   #$10
                TAY
                PLA                     ; GET SOURCE DATA
                AND   SIMA              ; LOGICAL AND

ALL             STY   SRC+1             ; SAVE FLAG SETTING DATA
                STA   SIMA              ; LOGICAL RESULT INTO A
                JSR   STATUS2           ; SET STATUS
                LDA   #$EE              ; SELECT STATUS BITS TO CLEAR
CYAC            AND   PSW               ; CLEAR SELECTED BITS
                ORA   SRC+1             ; SET SELECTED BITS
STPSW           STA   PSW               ; SAVE NEW PSW
                RTS

; ------------------------------------------------------------------------------
ROTATE          LDY   SIMA              ; GET ACCUMULATOR
                LSR                     ; IS THIS CMC OR 8TC?
                BNE   ROCOMP            ; NO, IT IS A ROTATE .OR CMA
                LDA   PSW               ; YES, CHANGE CARRY
                EOR   #$01              ; CMC
                BCC   STPS
                ORA   #$01              ; STC
STPS            BVS   STPSW

; ------------------------------------------------------------------------------
ROCOMP          BCS   LEFT              ; LEFT OR RIGHT?
                LSR                     ; CMA?
                BNE   ROT               ; NO, ROTATE
                TYA                     ; YES, COMMPLEMFNT A
                EOR   #$FF
                STA   SIMA              ; DOESN`T SET STATUS
                RTS

; ------------------------------------------------------------------------------
ROT             TYA                     ; GET ACCUMULATOR
                BCS   RRC
                LDA   PSW               ; GET 8080 CARRY
RRC             LSR
                TYA                     ; GET ACCUMULATOR
                ROR                     ; ROTATE RIGHT
                BVS   JCRY              ; GO STORE&SET CARRY

; ------------------------------------------------------------------------------
LEFT            LSR                     ; DECIMAL ADJUST? IYES# GO 00 IT
                BEQ   DAA               ; N 0 , GET ACCUMULATOR
                TYA                     ; JRLC?
                BCS   RLC               ; NO#GET 8080 CARRY
                LDA   PSW               ; MOVE IT INTO MSNIB
                ROR
                ROR
RLC             ASL                     ; MOVE IT INTO CARRY
                TYA                     ; G E T ACCUMULATOR
                ROL                     ; MOVE IT LEFT
JCRY            STA   SIMA              ; SAVE IT
                JMP   CARRY             ; GO SET STATUS

; ------------------------------------------------------------------------------
DAA             CLC
                PHP                     ; PRE3ERVE STATUS |
                TYA                     ; GET SOURCE DATA
                STA   ZERO              ; PREP FOR ADD
                AND   #$0F              ; LOOK AT LSNI8
                ADC   #$06              ; IF  *0A WILL CAUSE AUX CRY
                ORA   PSW               ; OR I F AC I S ALREADY SET
                AND   #$10              ; EITHER SET?
                BEQ   NOSIX             ; NO, DON  T ADJUST LSNIB
                LDA   #$06              ; YES# ADJUST IT
NOSIX           TAY
                ADC   ZERO              ; G E T SOURCE
                BCS   SXTY
                ADC   #$60              ; 113 MSNIB NOW >  A0T IIF SO* CARRY IS SET
                ROL                     ; G E T CARRY
                ORA   PSW               ; OR WITH 8080 CARRY
                LSR                     ; I S EITHER SETT
                BCC   DA1               ; NO# DON`T ADJUST MSNIB l
SXTY            TYA                     ; YES* ADJUST MSN!B I
                ADC   #$5F              ; (5F + CARRY = 60)
                TAY
DA1             LDX   #$00              ; DESTINATION IS A
                PLP                     ; RESTQRE STATUS
ADCRY           JSR   ADDR
                JSR   CARRY

STATUS2         TAY                     ; SAVE RESULT
                ROL   PSW               ; CLEAR P3W SIGN BIT
                ASL                     ; PUT NEW SIGN IN CARRY
                STA   FLAG              ; C L E A R L S B OF F L A G
                LDA   PSW               ; PUT NEW SIGN IN P3N
                ROR
SGN             ORA   #$46              ; PRESET Z*P 8 PRESET
                TAX                     ; 3AVE IN X
                TYA                     ; RECOVER WORD
                BEQ   DONE              ; I F ZERO*ALL DONE
FLIP            INC   FLAG              ; IFLIP FLAG |
PAR             LSR                     ; TEST EACH BIT I
                BEQ   ALL2              ; NO MORE BITS
                BCC   PAR
ALL2            BCS   FLIP
                LSR   FLAG              ; TEST FLAG
                TXA                     ; RECOVER P3w
                AND   #$BF              ; CLEAR Z
                BCS   REC               ; PARITY EVENT
                AND   #$FB              ; N0* CLEAR P
REC             TAX                     ; BACK TO X
DONE            STX   PSW               ; STORE AS PSW
                RTS

; ------------------------------------------------------------------------------
PCPNT           LDX   #$0A
RPPNT           LDY   #$11              ; 05 + 11 = 16 = store at DESTDA
                BNE   RPRP              ; never zero, branch always
RPSCR           LDY   #$13              ; store at SRC instead

; moves scratchpad or other RP bytes to PC, vice versa
RPRP            LDA   SIMA,X            ; load SOURCE BYTE L
                STA   SIMA,Y            ; store at DESTINATION L
                LDA   SIMM,X            ; load SOURCE BYTE H
                CPY   #$08
                BNE   RPRP4
                CLC
                ADC   REL               ; #$10
                JMP   RPRP2
RPRP4           CPX   #$0A              ; X=0A, PC IS POINTER.
                BNE   RPRP2
RPRP3           LDA   SIMM,X
RPRP2           STA   SIMM,Y            ; store at DESTINATION H
RPRTS           RTS

; ------------------------------------------------------------------------------
SCRRP           TXA                     ; RP is destination (RP is a Register Pair)
                TAY                     ; 0A on CALL (for return address???)
                LDX   #$13              ; SCR is source
                BNE   RPRP              ; never zero, branch always
SPPNT           LDX   #$08              ; SP is Source
                BNE   RPPNT             ; jump RPPNT
DBLINC          JSR   INCPC             ; ADVANCE COUNTER
INCPC           LDX   #$0A              ; INC PC - $0A + $05 = $0F = PCL
INCPC2          LDY   #$0E              ; point to INC data - $0E + $05 = $13 = INCIT (0001)
INCDEC          CLC
INDE            LDA   SIMA,X            ; get register/PC value
                ADC   SIMA,Y            ; add value from
                STA   SIMA,X            ; save
                INY                     ; bump indexes
                INX
                TYA                     ; check index 0E->0F
                AND   #$01              ; last pass? - increment HI byte
                BNE   INDE              ; not done, loop
                RTS                     ; yes

; ------------------------------------------------------------------------------
DECSP           JSR   DSP               ; DEC SP twice
DSP             LDX   #$08              ; DEC SP
                LDY   #$0C              ; point to DEC data
                BNE   INCDEC            ; go do it

; ------------------------------------------------------------------------------
MEMRP           TYA                     ; grab source pointer offset
                LDA   DESTDA+1
                LDY   #$01              ; move data in memory
MRLP            LDA   (DESTDA),Y        ; get next byte
                STA   SIMM,X            ; store in destination
                DEX                     ; bump indices
                DEY
                BEQ   MRLP
MRLPRTS         RTS

; ------------------------------------------------------------------------------
IMM             LDX   #$00              ; GET BYTE POINTED BY PROGRAM COUNTER
IMM2            LDA   (PCL,X)
                STA   SRC               ; SAVE IT IN SCR
                LDA   #$13              ; MAKE SCR THE SOURCE INDEX
                STA   SPEED             ; ??? should SPEED be SCR?
                JSR   INCPC             ; BUMP PC
                LDX   #$00              ; POINT TO INTERP PAGE 2
                RTS

; ------------------------------------------------------------------------------
ADDR            TYA
                EOR   FLAG
                TAY
                AND   #$0F
                STA   SRC
                ROL
                BVS   WOCRY
                EOR   PSW
WOCRY           PHA
                LSR
                LDA   ZERO
                AND   #$0F
                ADC   SRC
                AND   #$10
                STA   SRC+1
                LDA   #$EF
                JSR   CYAC
                PLA
                LSR
                TYA
                ADC   ZERO
                JMP   MVIT1

; ------------------------------------------------------------------------------
; TABLE OF PSW MASKS
MSKTBL          BYT   $80               ; SIGN BIT MASK
FOUR            BYT   $04,$01,$40       ; PARITY BIT MASK, CARRY BIT MASK, ZERO BIT MASK

; ------------------------------------------------------------------------------
; XFER TABLE
XFERTBL         BYT   >MOVIT,<MOVIT     ; MOVIT
                BYT   >ROTATE,<ROTATE   ; ROTATE
                BYT   >MVI,<MVI         ; MVI
                BYT   >DCR,<DCR         ; DCR
                BYT   >INR,<INR         ; INR
                BYT   >INXDCX,<INXDCX   ; INXDCX
                BYT   >LDSTR,<LDSTR     ; LDSTR
                BYT   >DADLXI,<DADLXI   ; DADLXI
                BYT   >NOP_,<NOP_       ; NOP
                BYT   >RST,<RST         ; RST
                BYT   >ARIM,<ARIM       ; ARIM
                BYT   >PUSH,<PUSH       ; PUSH
                BYT   >CALL,<CALL       ; CALL
                BYT   >IO,<IO           ; IO
                BYT   >JMP_,<JMP_       ; JMP
                BYT   >POP,<POP         ; POP
                BYT   >RETURN,<RETURN   ; RETURN
                BYT   >CMP_,<CMP_       ; CMP
                BYT   >ORA_,<ORA_       ; ORA
                BYT   >XRA,<XRA         ; XRA
                BYT   >ANA,<ANA         ; ANA
                BYT   >SBB,<SBB         ; SBB
                BYT   >SUB,<SUB         ; SUB
                BYT   >ADC_,<ADC_       ; ADC
                BYT   >ADD,<ADD         ; ADD

; ------------------------------------------------------------------------------
; I/O TABLE
; 00/10 = SIO STATUS = for left bank of switches
; 01/11 = SIO DATA   = to put a byte at lower left of screen

; 07    = Tape

; 08    = Disk Enable/Status channel
; 09    = Disk control/sector position channel
; 0A    = Disk data channel

; FF/0F = panel switches

IOTBLOUT        WBE(SIOSTATUSOUT)       ; 00
                WBE(SIODATAOUT)         ; 01
                WBE(DUMMY)              ; 02
                WBE(DUMMY)              ; 03
                WBE(DUMMY)              ; 04
                WBE(DUMMY)              ; 05
                WBE(DUMMY)              ; 06
                WBE(TAPEDATAOUT)        ; 07

                WBE(DSKSELECT)          ; 08
                WBE(DSKCONTROL)         ; 09
                WBE(DSKWRITE)           ; 0A
                WBE(DUMMY)              ; 0B
                WBE(DUMMY)              ; 0C
                WBE(DUMMY)              ; 0D
                WBE(DUMMY)              ; 0E
                WBE(DUMMY)              ; 0F

IOTBLIN         WBE(SIOSTATUSIN)        ; 10
                WBE(SIODATAIN)          ; 11
                WBE(DUMMY)              ; 12
                WBE(DUMMY)              ; 13
                WBE(DUMMY)              ; 14
                WBE(DUMMY)              ; 15
                WBE(DUMMY)              ; 16
                WBE(DUMMY)              ; 17

                WBE(DSKSTATUS)          ; 18
                WBE(DSKSECTORIN)        ; 19
                WBE(DSKREAD)            ; 1A
                WBE(DUMMY)              ; 1B
                WBE(DUMMY)              ; 1C
                WBE(DUMMY)              ; 1D
                WBE(DUMMY)              ; 1E
                WBE(DUMMY)              ; 1F

; ------------------------------------------------------------------------------
; SETUP 8080 REGISTERS
; ------------------------------------------------------------------------------
REGSETUP        LDX   #$00              ; ZERO
                STX   DSKSELECT
                STX   DSKTRACK
                STX   DSKSECTOR
                STX   SECTOROFFSET
                STX   TRACKPOINTER
                STX   TRACKPOINTER+1
                STX   TRKBUFPNT

                STX   PNT+$01           ; pointer Hi
                STX   PNT               ; pointer Lo
                STX   PCL               ; PC LO
                STX   PCH               ; PC HI ****

                STX   SCR               ; scratchpad
                STX   SIODATAIN         ; clear SIO DATA for use
                STX   SIODATAOUT        ; clear SIO DATA for use
                INX                     ; ONE
                STX   INTE              ; Interrupt enable = 1
                INX                     ; TWO
                STX   PSW               ; processor status = 00000010

                LDX   #$FF              ; $11-$12 == #$ff
                STX   DECIT
                STX   DECIT+$01
                STX   SPL
                STX   OLDTRACK

                INX
                STX   INCIT+$01         ; $13-$14 == #$00
                STX   HPOS              ; cursor to 0

                INX
                STX   INCIT             ; $13-$14 == 01 00

                LDX   #$0F
                STX   SPH

                RTS

; ------------------------------------------------------------------------------
DSKSPIN         LDX   DSKSECTOR         ; looking for a specific sector, but no way to tell which one
                INX                     ; increment DSKSECTOR through #$1F

                CPX   #$20              ; SECTOR 32
                BEQ   RSTTRKPNTR        ; set everything to 00

                STX   DSKSECTOR         ; set new DSKSECTOR
                TXA                     ; into ACCUMULATOR
                CLC

SETSECTORIN     ROL                     ; ROL and set as DSKSECTORIN
                STA   DSKSECTORIN       ; bit 0 always low == sector TRUE (ready to read/write)
                LDA   TRACKPOINTER      ; add 137 bytes to TRACKPOINTER
                CLC
                ADC   #$89              ; add 137
                STA   TRACKPOINTER
                BCC   DSKSPIN2
                INC   TRACKPOINTER+1
DSKSPIN2        LDA   #$00              ; reset to read/write at beginning of next sector
                STA   SECTOROFFSET
                RTS

RSTTRKPNTR      LDA   TRACKBUFFERLO
                STA   TRACKPOINTER
                LDA   TRACKBUFFERHI
                STA   TRACKPOINTER+1
                LDA   #$00
                STA   DSKSECTOR
                STA   DSKSECTORIN
                STA   SECTOROFFSET
                RTS

; ------------------------------------------------------------------------------
; GET DATA FROM VIRTUAL DSK TRACK
; ------------------------------------------------------------------------------
DSKIN           LDY   SECTOROFFSET      ; which byte to grab
                LDA   TRACKPOINTER
                STA   $00
                LDA   TRACKPOINTER+1
                STA   $01
                LDA   ($00),Y           ; grab byte from track pointer + offset
                STA   DSKREAD
                INC   SECTOROFFSET      ; ready for next read
                RTS

; ------------------------------------------------------------------------------
; VIRTUAL SERIAL INTERFACE - JUST GOT A BYTE FROM SIM80
; ------------------------------------------------------------------------------
SIOOUT          LDA   SRC               ; if output is 00, setting status.
                BEQ   PUNCHOUT          ; 00 - punch device

                AND   #$0F              ; 1x => 0x
                CMP   #$01              ; is it on port 1 or 11?
                BEQ   SIOOUT2           ; yes, sio out
                JMP   DSKOUT            ; if not, is it output to DSK?

SIOOUT2         LDA   SIODATAOUT        ; LOAD BYTE FROM SIODATA
                CMP   #$0A              ; translate CR/LF into newline.
                BEQ   SIORTS            ; ignore LF, just do CR.
                ORA   #$80              ; add the high bit?
                JSR   ROM_COUT          ; output char

SIORTS          SEC
                RTS

; ------------------------------------------------------------------------------
PUNCHOUT        JMP   SIORTS            ; TODO: ....

; ------------------------------------------------------------------------------
JMPTAPEIN       JMP   TAPEIN            ; TODO: ....

; ------------------------------------------------------------------------------
JMPDSKSPIN      JMP   DSKSPIN           ; spaghetti

; ------------------------------------------------------------------------------
; SIGNALLED FROM IOIN THAT SIM80 LOOKING FOR A BYTE.
SIOIN           LDX   #$00              ; looking for a byte from IN 1 or 0?
                LDA   SRC               ; if input is 00, checking SIO status.

                CMP   #$FF              ; sense switches.
                BEQ   SENSEIN

                CMP   #$0A              ; looking for DISK DATA
                BEQ   DSKIN

                CMP   #$09              ; looking for DISK SECTOR
                BEQ   JMPDSKSPIN        ; "spin" the disk

                CMP   #$06
                BEQ   JMPTAPEIN
                CMP   #$07
                BEQ   JMPTAPEIN

CHKSIO          CMP   #$10              ; SIO2 on port 10/11
                BCC   SIO1IN            ; SIO1 on port 00/01
                INX
                INX                     ; use SIO2 status bytes instead of SIO1
                JMP   SIO2IN

SIO1IN          AND   #$0F              ; kill hi nibble so 10==00
                BNE   SIOKEY            ; 00? status check. Otherwise, expects a byte.
                STA   SIODATAIN         ; clear the pipes on status check, 00 to data
                JMP   SIOREADY          ; set status to ready for I/O

SIOKEY          JSR   SIORESET          ; HACK CHECK KEYBOARD
                DEX                     ; HACK
                JSR   ROM_GETKEY        ; read keyboard

; IF KEY WAITING, LOAD IT IN SIODATA
                BPL   SIORTS            ; no key. RTS.
;               BPL   CHECKKSW          ; if bit8/carry, has keyboard input. else check for BASIC LOAD
                STA   SIODATAIN         ; put ascii in DATA

; SET SIOSTATUS TO 00 = HAVE DATA
                LDA   SIOSTATUSMASK,X   ; set STATUS to 00 - HAVE DATA/READY FOR OUTPUT
                STA   SIOSTATUSIN
                RTS

; NO KEY WAITING - SET SIOSTATUS 01 = STILL WAITING INPUT
; CHECKKSW      JSR   KSW             ; CHECK FOR KSW INPUT FROM BASIC LOAD
; should only run on input and if src = 1?
;               BPL   SIORESET        ; if high bit set, have a byte from BASIC LOAD. otherwise, return

SIOREADY        ; SET SIOSTATUS TO 00 = HAVE DATA
                LDA   SIOSTATUSMASK,X   ; set STATUS to 00 - HAVE DATA/READY FOR OUTPUT
                STA   SIOSTATUSIN
                RTS

SIORESET        INX
                LDA   SIOSTATUSMASK,X   ; 01 = WAITING FOR INPUT/READY FOR OUTPUT
                STA   SIOSTATUSIN
                RTS

SENSEIN         AND   #$0F
;               CLC                     ; NOTE: optimized
;               ROL
                ASL
                TAY
                LDA   IOTBLIN,Y         ; sense switches up/down don`t have an IN/OUT version
                STA   DESTDA+1
                INY
                LDA   IOTBLIN,Y
                STA   DESTDA
                RTS

SIO2IN          AND   #$0F              ; #$10 -> #$00

; FIXME: do we need different handling?
; # FIXED       BEQ   SIO2STATUS        ; SIO2 data input
; # FIXED       JSR   ROM_GETKEY        ; get the key
; # FIXED       STA   SIODATAIN         ; store it in datain buffer
; # FIXED       JMP   SIORESET          ; input done. set status to waiting for next check.
; # FIXED SIO2STATUS                    ; looking for status of SIO2 - have a byte, byte sent, or 00 waiting

                JSR   ROM_GETKEY        ; if ROM_GETKEY bit 8 hi, have a byte
                BPL   SIORESET          ; bit8 lo, set status to byte sent/waiting
                STA   SIODATAIN         ; NOTE: ADDED store it in datain buffer
                JMP   SIOREADY          ; bit 8 hi, has keyboard input.

; ------------------------------------------------------------------------------
DSKOUT          CMP   #$08              ; port 08 -> select disk
                BNE   DSKCONTR
;@              LDA   DSKSELECT
;@              LDA   #$FF
;@              STA   OLDTRACK
                JMP   OUTRTS

DSKCONTR        CMP   #$09              ; port 09 -> disk control
                BNE   DSKOUT2
;@              LDA   DSKCONTROL
                JMP   OUTRTS

DSKOUT2         CMP   #$0A              ; port 0A -> output to DSK
                BNE   TAPEOUT           ; not 0A? -> ignore

                LDY   SECTOROFFSET      ; which byte of the current sector to push to
                LDA   TRACKPOINTER      ; offset within track ($89xSECTOR)
                STA   $00
                LDA   TRACKPOINTER+1
                STA   $01
                LDA   DSKWRITE          ; load the byte
                STA   ($00),Y           ; set byte at track pointer + offset
                INC   SECTOROFFSET      ; ready for next read
                RTS

; ------------------------------------------------------------------------------
TAPEOUT         CMP   #$07              ; tape?
                BNE   OUTUNKNOW
                LDA   TAPEDATAOUT       ; TODO: store tape data
                SEC
                RTS

; ------------------------------------------------------------------------------
OUTUNKNOW     ; STA   ODEBUG            ; ouput debug info

OUTRTS          SEC                     ; one byte < JMP SIORTS
                RTS


; ------------------------------------------------------------------------------
TAPEIN          RTS

; ------------------------------------------------------------------------------
; FIXME: tape routines ?
START_TAPE      LDA   TRACKBUFFERLO     ; locate buffer start
                STA   $CE
                LDA   TRACKBUFFERHI     ;
                STA   $CF

KSW             STY   $CD               ; SAVE Y
                LDY   TRKBUFPNT         ; byte counter
                LDA   ($CE),Y           ; get byte from buffer
                BEQ   KILLKSW           ; if it`s EOF (00) set keyboard control back

                LDA   SRC               ; if looking for input byte on 01/11 ?
                AND   #$0F              ; kill hi nibble
                CMP   #$01
                BNE   SETHIBIT          ; otherwise, set hi bit to say "data ready"

                LDA   ($CE),Y           ; get byte AGAIN?
                ORA   #$80              ; sets high bit of incoming text
                STA   SIODATAIN         ; returns with high bit set...

                INC   TRKBUFPNT         ; next pointer address
                BNE   SETHIBIT
                INC   $CF               ; rolled over, get next hi byte

SETHIBIT        LDY   $CD               ; RESTORE Y
                ORA   #$80              ; sets high bit of incoming text
                RTS

KILLKSW         LDY   $CD               ; RESTORE Y
                LDA   #$00              ; defeats BPL
                RTS

; ------------------------------------------------------------------------------
; FIXME: needed ???
PCHFIX          CLC                     ;@FIX
                LDA   PCH               ;@FIX
                ADC   REL               ;@FIX
                STA   PCH               ;@FIX
                LDA   (PCL),Y           ;@FIX
                STA   OPCODE            ;@FIX
                SEC                     ;@FIX
                LDA   PCH               ;@FIX
                SBC   REL               ;@FIX
                STA   PCH               ;@FIX
                RTS                     ;@FIX

; ------------------------------------------------------------------------------
DUMPREGS        LDA   VPOS
                PHA
                LDA   HPOS
                PHA

                LDX   #$00
                LDY   #$00
                STY   HPOS              ; back to left side to

                CLC
                LDA   PCH
                ADC   REL               ; #$10
                STA   PCH

                LDA   (PCL),Y           ; print registers values
                STA   OPCODE

                SEC
                LDA   PCH
                SBC   REL               ; #$10
                STA   PCH

                LDA   #WNDH             ; move cursor to line 22
                JSR   ROM_BASCALC
                LDA   PSW
                JSR   PRREG             ; PS = Processor Status byte
                LDA   SIMA
                JSR   PRREG             ; AC = Accumulator
                LDA   SIMB
                LDX   SIMC
                JSR   PRLOHI            ; BC = B & C Registers
                LDA   SIMD
                LDX   SIME
                JSR   PRLOHI            ; DE = D & E Registers
                LDA   SIMH
                LDX   SIML
                JSR   PRLOHI            ; HL = H and L Registers
                LDA   SPH
                LDX   SPL
                JSR   PRLOHI            ; SP = Stack Pointer HiLo
                LDA   PCH
                LDX   PCL
                JSR   PRLOHI            ; PC = SIM80 Program Counter Hi and Lo
                LDA   OPCODE            ; OP = current opcode
                JSR   PRREG
                PLA
                STA   HPOS
                PLA
                JMP   ROM_BASCALC       ; J/R
;               RTS

; ------------------------------------------------------------------------------
PRLOHI          JSR   ROM_PRBYTE        ; prints a pair of registers
                TXA
PRREG           JSR   ROM_PRBYTE        ; print hex of one register, followed by space (#$A0)
                LDA   #$20
                JMP   ROM_COUT          ; JMP to ROM_COUT with RTS ?

; ------------------------------------------------------------------------------
COPYRIGHT       BYT   "APPLE-80 COPYRIGHT 1979 by DANN MCCREARY"
                BYT   "     SIM8800 by CHARLES MANGIN, 2019    "
                BYT   "                                        "
                BYT   "                                        "
                BYT   ":                                      :"
                BYT   "     ORIC PORT by      .__              "
                BYT   "  ____________  ___  __|__| _____ _____ "
                BYT   "  \_  __ \__  \ \  \/  /  |/ ___// ___/ "
                BYT   "   |  | \_/ __ \_>    <|  |\__ \ \__ \  "
                BYT   "   |__|  (____  /__/\_ \__/___  >___  > "
                BYT   "              \/      \/      \/    \/  "
                BYT   ":                                      :"
                BYT   "                                        "
                BYT   $00

; ------------------------------------------------------------------------------
HELLO           JSR   CURSOROFF
                LDA   #<COPYRIGHT
                STA   $00
                LDA   #>COPYRIGHT
                STA   $01
                LDA   #<WNDA
                STA   $02
                LDA   #>WNDA
                STA   $03

                LDY   #$00
HELOOP          LDA   ($00),Y
                BEQ   HELLORET
                ORA   #$80
                STA   ($02),Y
                INY
                BNE   HELOOP
                INC   $01
                INC   $03
                BNE   HELOOP
HELLORET        RTS

; ==============================================================================
; ERROR           JSR   ROM_PRBYTE        ; Print error code
;                 JSR   ROM_CROUT         ; Print a carriage return
;                 JSR   ROM_BELL
;                 RTS

; ==============================================================================

SETDSKOFFSET    LDA   #$00
RESETOFFSET     STA   DSKOFFSET         ; rezero the offset
                STA   DSKOFFSET+1
                STA   DSKOFFSET+2
                LDX   DSKTRACK          ; how many loops to INC offset by?
                BEQ   SETMARK          ; zero = skippity

MARKLOOP        LDA   DSKOFFSET         ; should be zero to start magic happens
                CLC
                ADC   #$20              ; multiply $1120 by track number
                STA   DSKOFFSET
                LDA   DSKOFFSET+1       ; store result in DSKOFFSET
                ADC   #$11
                STA   DSKOFFSET+1
                BCC   MARKLOOP2
                INC   DSKOFFSET+2
MARKLOOP2       DEX                     ; magic happens
                BNE   MARKLOOP

SETMARK         LDA   #SETMARKCMD
                JMP   ROM_DSKIO
;@              BNE   ERROR
;@              RTS

READDSK         JSR   SETDSKOFFSET      ; SETMARK
                LDA   #READCMD
                JMP   ROM_DSKIO
;@              BNE   ERROR             ; error if return !=0
;@              RTS

WRITEDSK        JSR   SETDSKOFFSET      ; SETMARK
                LDA   #WRITECMD
                JMP   ROM_DSKIO
;@              BNE   ERROR             ; error if return !=0
;@              RTS

DSKOFFSET       BYT   $00,$00,$00       ; 3 bytes, lo to hi

; ==============================================================================
; Defaults     ; DSKTRACK == 0
; set DSKSTATUS = A5   1 == FALSE, 0 == TRUE
; bit 0 = not ready to write  1
; bit 1 = head move allowed  0
; bit 2 = head status not on 1
; bit 5 = interrupt disabled = 1
; bit 6 = 0 - on track 0
; bit 7 = 1 - data not ready yet

; set DSKSECTORIN = 0
; on DSKCONTROL out - if bit 2 = 1, then load 137 bytes from DSKTRACK.
; OPEN DISKBASIC41, SETMARK = DSKTRACK, READ 32*137 bytes into TRACKBUFFER

; ==============================================================================

DSKWRITE        BYT   $00               ; port 0A out - data to write
DSKREAD         BYT   $00               ; port 0A in - data to read

DSKSELECT       BYT   $00               ; port 08 out - latches and enables drive - ignored for now, only enabling 1 drive (0)
DSKSTATUS       BYT   $A5               ; port 08 in - indicates drive status 1010 0101  0=TRUE, 1=FALSE not ready to write, move enable, head off, n/a, n/a, interrupt disable, on track 0, no data
DSKCONTROL      BYT   $00               ; port 09 out - controls drive functions  1=TRUE, 0=FALSE
DSKSECTORIN     BYT   $00               ; port 09 in - indicates disk sector position - start at 00 0-31 offset by 1 bit, bit 0=0 when on sector 0, 1 otherwise.

SECTOROFFSET    BYT   $00               ; how far into the sector to grab bytes
TRACKPOINTER    BYT   $00,$00           ; how far into the track to look for sector start

; ==============================================================================

SIMDSK          LDA   DSKCONTROL        ; see what the last operation wants from the drive
                BEQ   DSKRTS            ; ZERO? Do nothing.
                TAX                     ; hang onto DSKCONTROL byte

                AND   #$03              ; one/two LO bits set? 00000011
                BNE   MOVEHEAD          ; moving head in/out

                TXA                     ; get DSKCONTROL back
                CMP   #$08              ; bits 2/3 set, load/unload head 00001000
                BEQ   UNLOADHEAD

                CMP   #$80
                BEQ   WRITESEQUENCE     ; == #$80 for DSK write???

                AND   #$04              ; == 04 TRUE = load HEAD
                BEQ   DSKRTS            ; anything else?

LOADHEAD        LDA   DSKSTATUS         ; if bit 0=0 then, yes. Write was initiated at least.
                TAX
                AND   #$01              ; if it`s 1, no need to write
                BNE   LOADHEAD2

                TXA                     ; otherwise, write the track to DSK file
                ORA   #$01
                STA   DSKSTATUS         ; set bit 0 to disable write for next pass

                JSR   WRITEDSK          ; write the track

LOADHEAD2       JSR   READDSK          ; LOAD TRACK TO BUFFER AFTER WRITING CURRENT TRACK BACK TO DISK

                LDA   DSKSTATUS         ; DSKCONTROL == #$04
                AND   #$7B              ; 0111 1011B .....0.. = head on, data waiting
;               ORA   #$02              ; 0000 0010B head not movable - unnecessary

                STA   DSKSTATUS         ; move off, head on,

                JMP   DSKRTS

UNLOADHEAD      LDA   DSKSTATUS         ; DSKCONTROL == #$08
                ORA   #$04              ; 00000100B     .....10. = move on, head off
                AND   #$FD              ; 11111101B
                STA   DSKSTATUS

DSKRTS          LDA   #$00
                STA   DSKCONTROL        ; reset DKSCONTROL to prevent loopity
                RTS

WRITESEQUENCE   LDA   DSKSTATUS         ; unset bit 0 to enable write
                AND   #$FC              ; set head loaded true. STATUS bit 2 = 0
                STA   DSKSTATUS         ; 11111100

; set sector status true. SECTOR bit 0 = 0
; bytes OUT on 0A

                JMP   DSKRTS

MOVEHEAD        ; DID WE WRITE TO THIS TRACK?
                LDA   DSKSTATUS         ; if bit 0=0 then, yes. Write was initiated at least.
                TAX
                AND   #$01              ; if it`s 1, no need to write
                BNE   MOVEHEAD2

                TXA                     ; otherwise, write the track to DSK file
                ORA   #$01
                STA   DSKSTATUS         ; set bit 0 to disable write for next pass

                JSR   WRITEDSK          ; write the track

MOVEHEAD2       LDA   DSKCONTROL        ; move which way?
                ROR                     ; bit 0 into carry
                BCS   STEPIN            ; bit 0 set, move to HIGHER track #

STEPOUT         LDA   DSKTRACK          ; if already on track 0, ignore.
                BEQ   SETTRACK0         ; make sure TRACK0 bit is set?
                DEC   DSKTRACK          ; otherwise next lower track
                BEQ   SETTRACK0
                JSR   READDSK          ; LOAD TRACK N TO BUFFER
                JMP   DSKRTS

SETTRACK0       LDA   DSKSTATUS
                AND   #$BF              ; 10111111 unset bit 6 (track 0=TRUE)
                STA   DSKSTATUS
                JSR   READDSK          ; LOAD TRACK 0 TO BUFFER
                JMP   DSKRTS

STEPIN          LDA   DSKTRACK
                CMP   #$4D              ; can`t go higher than 77
                BEQ   DSKRTS
                INC   DSKTRACK
                LDA   DSKSTATUS
                ORA   #$40              ; 01000000 - sets bit 6, not on TRACK0
                STA   DSKSTATUS
                JSR   READDSK          ; LOAD TRACK N TO BUFFER
                JMP   DSKRTS

; ==============================================================================
; PCH/REGD PCL OPCODE

SIOSTATUSOUT    BYT   $00               ; probably unnecessary
SIOSTATUSIN     BYT   $81               ; not ready for IO

SIODATAOUT      BYT   $00               ; Virtual Serial Interface, status buffer, DATA buffer
SIODATAIN       BYT   $00               ; Virtual Serial Interface, status buffer, DATA buffer

SIOSTATUSMASK   BYT   $00,$01           ; status settings for "Have data" and "waiting data"
SIO2STATUSMASK  BYT   $01,$02           ; has a byte, last byte sent

TAPESTATUSOUT   BYT   $00
TAPEDATAOUT     BYT   $00

DUMMY           BYT   $00               ; not ready for IO

; SIO2
; Receive Data Register Full (RDRF), Bit 0 - Receive Data Register Full indicates
; that received data has been transferred to the Receive Data Reqister. RDRF is
; cleared after a read of the Receive Data Register or by a maiter reset.
; The cleared or empty state indicates that the contents of the Receive Data Register are not current.

; Transmit Data Register Empty (TDRE), Bit 1 - The Transmit Data Register Empty bit being set high
; indicates that the Transmit Data Register contents have been transferred and that new data may be entered.
; The low state indicates that the register is full and that transmission of a new character has not begun
; since the last write data command.

TRACKBUFFERLO   BYT   <TRACKBUFFER
TRACKBUFFERHI   BYT   >TRACKBUFFER
TRKBUFPNT       BYT   $00
